import java.util.ArrayList;
import java.util.List;

/**
 * A group of students.
 */

public class Cohort {
  private final String name;
  private final List<Student> students;

  /**
   * Constructs a cohort with a name equals to the specified {@code name} and no students.
   * @param name the name of the constructed Cohort
   */

  public Cohort(String name) {
    this.name = name;
    this.students = new ArrayList<>();
  }

  /**
   * Add the specified {@code student} to the students of the cohort.
   * @param student the student to be added to the cohort
   */
  public void addStudent(Student student){
    this.students.add(student);
  }

  /**
   * Returns the list of students of the cohort.
   * @return the list of students of the cohort.
   */
  public List<Student> getStudents(){
    return students;
  }

  /**
   * Print via the standard output the name of the cohort and all results associated to the students with their average
   * grade.
   */
  public void printStudentsResults(){
    printName();
    for (Student student : students){
      student.printResults();
    }
    System.out.println("Le nombre d'étudiants ayant validé leur année : " + getPass() +"/" + students.size());
    System.out.println("L'étudiant avec la note la plus élevée : " + maxAverageName() + " avec une moyenne de " + maxAverage() + "/20");
    System.out.println("L'étudiant avec la note la plus basse : " + minAverageName() + " avec une moyenne de " + minAverage() + "/20");
    System.out.println("La moyenne de la classe est de : " + getAverageOfClass() + "/20");

  }

  private void printName(){
    System.out.println(this.name);
  }

  /**
   * Returns the name of the cohort.
   * @return the name of the cohort
   */
  @Override
  public String toString() {
    return this.name;
  }
  public int getPass(){
    int pass = 0;
    for (Student student : students){
      if (student.getAverageGrade().getValue() >= 10){
        pass += 1;
      }
    }
    return pass;
  }
  public double maxAverage(){
    double max = 0;
    for (Student student : students){
      if (student.getAverageGrade().getValue() >= max){
        max = student.getAverageGrade().getValue();
      }
    }
    return max;
  }
  public String maxAverageName(){
    double max = 0;
    String name = "";
    for (Student student : students){
      if (student.getAverageGrade().getValue() >= max){
        max = student.getAverageGrade().getValue();
        name = student.toString();
      }
    }
    return name;
  }
  public double minAverage(){
    double min = 21;
    for (Student student : students){
      if (student.getAverageGrade().getValue() < min){
        min = student.getAverageGrade().getValue();
      }
    }
    return min;
  }
  public String minAverageName(){
    double min = 21;
    String name = "";
    for (Student student : students){
      if (student.getAverageGrade().getValue() < min){
        min = student.getAverageGrade().getValue();
        name = student.toString();
      }
    }
    return name;
  }
  public double getAverageOfClass(){
    int number = 0;
    double sum = 0;
    double avg;
    for (Student student : students){
      number += 1;
      sum += student.getAverageGrade().getValue();
    }
    avg = sum / number;
    return avg;
  }
}
