import java.util.ArrayList;
import java.util.Arrays;
import java.util.Scanner;
import java.io.*;

/**
 * Class Crossword : Creates a 2D array of characters read from the input file
 */

public class Crossword {
    private char array[][];    // Tableaux 2D contenant les caractères
    private int rows;        // Nombres de lignes de array
    private int columns;        // Nombres de colonnes de array

    private int WordCount;   // Nombre d'occurences d'un mot
    private int PositionX;    // Colonne de début du mot
    private int PositionY;    // Ligne de début du mot
    private int EndX;        // Colonne de fin du mot
    private int EndY;        // Ligne de fin du mot


    /*** Constructor: Reads each line from File <file> and writes to a new row in the array.
     ** Updates <rows> and <columns> to height and width of the array. */
    public Crossword(File file) throws IOException {

        if (!file.exists()) {
            System.out.println("le fichier n'existe pas");
            return;
        }
        // Code pour calculer la taille du tableau et initialiser les variables d'instance "rows" et "columns"

        Scanner scan = new Scanner(file);

        columns = scan.next().length();
        while (scan.hasNextLine()) {
            rows++;
            scan.nextLine();
        }

        // Code pour créer le tableau "array"
        char[][] array = new char[rows][columns];

        // Code pour remplir "array" avec les caractères du Fichier "file"
        Scanner scanner = new Scanner(file);
        for (int i = 0; i < rows; i++) {
            String fileLine = scanner.nextLine();
            for (int j = 0; j < columns; j++) {
                char nextChar = fileLine.charAt(j);
                array[i][j] = nextChar;
            }
        }
        this.array = array;

    }


    /*** Methode Search(String) : Trouver le premier occurence du mot <word> dans le Tableau
     *** Si touver, mettre à jour les valeurs (PositionY, PositionX, EndY, EndX)*/

    public boolean search(String word) {
        // Verifier que le taile du mot "word" est supérieure à zero. Sinon rien à faire.
        if (word.length() <= 0) {
            return false;
        }
        for (int j = 0; j < columns; ++j) {
            for (int i = 0; i < rows; ++i) {
                if (searchRow(j, i, word)) {
                    PositionX = j;
                    EndX = PositionX + word.length() - 1;
                    PositionY = i;
                    EndY = PositionY;
                    //System.out.println(PositionX + " " + EndX + " " + PositionY +  " " + EndY);
                    return true;
                } else if (searchColumn(j, i, word)) {
                    PositionX = j;
                    EndX = PositionX;
                    PositionY = i;
                    EndY = PositionY + word.length() - 1;
                    //System.out.println(PositionX + " " + EndX + " " + PositionY +  " " + EndY);
                    return true;
                } else if (searchDiagonalRight(j, i, word)) {
                    PositionX = j;
                    EndX = PositionX + word.length() - 1;
                    PositionY = i;
                    EndY = PositionY + word.length() - 1;
                    //System.out.println(PositionX + " " + EndX + " " + PositionY +  " " + EndY);
                    return true;
                } else if (searchDiagonalLeft(j, i, word)) {
                    PositionX = j;
                    EndX = PositionX - word.length() + 1;
                    PositionY = i;
                    EndY = PositionY + word.length() - 1;
                    //System.out.println(PositionX + " " + EndX + " " + PositionY +  " " + EndY);
                    return true;
                }

            }

        }
        return false; // mot pas trouvé
    }

    /* Methode Interne SearchRow(int,int, String) : Cherche une ligne du tableaux pour le mot <word> à partir de array[y][x] */

    private boolean searchRow(int y, int x, String word) {
        int length = word.length();
        String wordfound = "";
        for (int z = 0; z < length; ++z) {
            if (y < this.columns) {
                if (array[x][y] == word.charAt(z)) {
                    wordfound = wordfound + word.charAt(z);
                    ++y;
                }
            }
        }
        return (wordfound.equals(word));
    }

    /* Methode Interne SearchRow(int,int, String) : Cherche une colonne du tableaux pour le mot <word> à partir de array[y][x] */

    private boolean searchColumn(int y, int x, String word) {
        int length = word.length();
        String wordfound = "";
        for (int z = 0; z < length; ++z) {
            if (x < this.rows) {
                if (array[x][y] == word.charAt(z)) {
                    wordfound = wordfound + word.charAt(z);
                    ++x;
                }
            }
        }
        return (wordfound.equals(word));
    }

    private boolean searchDiagonalRight(int y, int x, String word) {
        int length = word.length();
        String wordfound = "";
        for (int z = 0; z < length; ++z) {
            if (y < this.columns) {
                if (x < this.rows) {
                    if (array[x][y] == word.charAt(z)) {
                        wordfound = wordfound + word.charAt(z);
                        ++y;
                        ++x;
                    }
                }
            }
        }
        return (wordfound.equals(word));
    }

    private boolean searchDiagonalLeft(int y, int x, String word) {
        int length = word.length();
        String wordfound = "";
        for (int z = 0; z < length; ++z) {
            if (y >= 0) {
                if (x < this.rows) {
                    if (array[x][y] == word.charAt(z)) {
                        wordfound = wordfound + word.charAt(z);
                        --y;
                        ++x;
                    }
                }
            }
        }
        return (wordfound.equals(word));
    }

    /*** Methode pour visualiser le tableau. (Déjà fourni) */
    public void display() {
        if (rows > 0 && columns > 0)
            CrosswordGUI.display(array);
        else
            System.out.println("Error: Array is Empty.");
    }

    /*** Methode pour visualiser le tableau avec le mot en surbrillance. (Déjà fourni) */
    public void displayWord() {
        if ((PositionX < 0) || (PositionX > EndX) || (EndX >= columns)) {
            System.out.println("Error: Incorrect x-coordinates for Word");
            return;
        }
        if ((PositionY < 0) || (PositionY > EndY) || (EndY >= rows)) {
            System.out.println("Error: Incorrect y-coordinates for Word");
            return;
        }
        CrosswordGUI.display(array, PositionY, PositionX, EndY, EndX);
    }

    public void printArr() {
        for (int j = 0; j < columns; j++) {
            for (int i = 0; i < rows; i++)
                System.out.print(array[i][j] + "");
        }
        System.out.println();
    }

    public int countWord(String word) {
        this.WordCount = 0;
        for (int j = 0; j < columns; ++j) {
            for (int i = 0; i < rows; ++i) {
                if (searchRow(j, i, word)) {
                    this.WordCount++;
                } else if (searchColumn(j, i, word)) {
                    this.WordCount++;
                } else if (searchDiagonalRight(j, i, word)) {
                    this.WordCount++;
                } else if (searchDiagonalLeft(j, i, word)) {
                    this.WordCount++;
                }
            }
        }
        return this.WordCount;
    }

}

